/************************************************************************************/
/* Initialization procedure for sreLite.80  */
/* this file MUST be named srelite0.cmd  */
sre_lite_init:

parse arg thisaddr,port,verbose,accept_range,reset

call init1
if reset=1 then signal step1


/**************** Advanced user changeable parameters *******/
/*      To reset these, you MUST restart GoServe           **/

/* of particular interest are the EMPTY_PATH_OK and USE_SRELITE_RXL
parameters */

/* The sre-http addon directory. If 0, use \ADDON of GoServe directory */
addon_dir=0

/* The cgi-bin directory.  If 0, use \CGI-BIN of GoServe directory */
cgi_bin_dir=0

/* fully qualified name of common-log file. 0 means sreLite.log
   in the goserve directory */
common_log_file=0

/* set to 1 to enable digest authentication, 0 for basic authentication only */
digest_auth=1

/* set to 1 to enable support for empty cgi-bin "paths", 0 to disable*/
empty_path_ok=1

/* use http/1.1 response protocol. 
   sreLite supports a minimal version of the http/1.1 protocol.
   To enable http/1.1, set enable_http11=1.
   Otherwise, sreLite will report itself as http/1.0 
   Actually, in some odd cases sreLite will report itself as http/1.0,
   (even when enable_http11=1) -- but these are usually just for error
   messages, etc.
*/
enable_http11=1

/* map extensions to differnt command interpreters for cgi-bin scripts. 
 In particular, the PERL interpreter.
 This should be a space delimited list, with each item having
 the syntax:  
    ext=progname
 where
    ext: the extension of a script
    progname: the program name (presumably in your PATH) of the interpreter
              to be used with this script
 The classic example is: PL=PERL5 -- which means "all scripts ending
 in a .PL are to be interpeted by the PERL5 interpreter" (which
 we assume has been installed on th server machine).
*/
interpret_types=' PL=PERL5 ' 

/* Allow http/1.1 proxies to cache responses
   0=disallow
   1= allow
   2= allow, but force revalidation by all caches
   3= allow, but force revaliation by shared (proxy) caches
Note: this does NOT effect the GoServe cache */
proxy_cache=1


/* Suppress the VARSTORE thread. The VARSTORE thread provides a 
  global environment for storing variables -- allowing information
  to be shared across requsts, and retained over time.  
  It is not required by SRE-lite, but some addons may use it.
  You can save a little bit system resources by not launcing it.
  Set suppress_varstore=1 to not launch this thread, 0 to launch */
suppress_varstore=0

/* the temporary work directory (sreLite uses this for temporary work space 
   If 0, use \TEMP of GoServe directory */
tempdata_dir=0

/* the temporary WWW files directory (script and addons can use this for
   temporary work space). If 0, use \TEMP directory of the GoServe "data"
   directory */
tempfile_dir=0

/* file containing usernames/passwords. If 0, use data\users.in under goserve 
   directory */
user_file=0

/* if 1, check for  "directory request that does not end with a /"
   For example:  /myfiles/input instead of /myfiles/input/
   If found, the add a / and redirect (to this "with /" selector)
   If 0, do not check 
*/
add_slash=1



/**********************End of User Changeable parameters *************************/


foo=value("sreLite_INIT",0,'os2environment')

use_srelite_rxl=1               /* force use of srelitep.rxl */


/*---   Load REXX libraries ----- */
/* Load up advanced REXX functions */
foo=rxfuncquery('sysloadfuncs')
if foo=1 then do
  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  call SysLoadFuncs
end
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 call rxfuncadd 'rexxlibregister','rexxlib', 'rexxlibregister'
 call rexxlibregister
end
if rxfuncquery('sref_fileread')<>0 then do
  ik=rxfuncadd('sref_fileread','rexxlib','LIB_FILEREAD')
  if ik<>0 then call pmprintf('|sreLite: Warning: unable to register SREF_FILEREAD')
end
if rxfuncquery('sref_filewrite')<>0 then do
  ik=rxfuncadd('sref_filewrite','rexxlib','LIB_FILEWRITE')
  if ik<>0 then call pmprintf('|sreLite: Warning: unable to register SREF_FILEWRITE')
end

candogzip=1
if rxfuncquery('rxgzLoadFuncs')=1  then do
  foo=RXFuncAdd( 'rxgzLoadFuncs', 'rxgzLIB', 'rxgzLoadFuncs')
  if foo=0 then call rxgzLoadFuncs
end
if rxfuncquery('rxgzLoadFuncs')=1  then do
   candogzip=0
end



/* load srefprc.rxl */
if use_srelite_rxl<>1 then do
  lib_dir=strip(servdir,'t','\')||'\LIB'
  aa=value('SRE_LIBDIR',lib_dir,'os2environment')
  if macroquery('SREF_ERROR')="" then do  /* load rxl library? */
     call pmprintf("|sreLite: Loading SREFPRC.RXL into macrospace ")
     rxlfile=lib_dir||'\SREFPRX.RXL'
     aa=macroload(rxlfile)
     if aa=0 then do
        call pmprintf("|sreLite ERROR: "|| rxlfile ||" not available! "|| aa)
        'STRING Error, missing 'rxlfile
        exit
     end
  end
end
else do
  lib_dir=strip(servdir,'t','\')
  if macroquery('SRELI_PROCESS_ADDON')="" then do  /* load rxl library? */
     call pmprintf("|sreLite: Loading SRELITEP.RXL into macrospace ")
     rxlfile=lib_dir||'\SRELITEP.RXL'
     aa=macroload(rxlfile)
     if aa=0 then do
        call pmprintf("|sreLite ERROR: "|| rxlfile ||" not available! "|| aa)
        'STRING Error, missing 'rxlfile
        exit
     end
  end
end /* do */

aa=sref_version(,1)
if aa<130302 then do
   call pmprintf('sreLite ERROR: incorrect version of macrospace library ('aa') detected')
   'STRING Server Error: incorrect version of procedure library'
   exit
end

if cgi_bin_dir=0 then cgi_bin_dir=servdir||'\CGI-BIN'
if dosisdir(cgi_bin_dir)=0 then do
  say "|sreLite ERROR: no such cgi-bin directory: " cgi_bin_dir
  "string ERROR: no such cgi-bin directory: " cgi_bin_dir
   exit
end
cgi_bin_dir=strip(strip(cgi_bin_dir),'t','\')
foo=value('SRELI_CGIBIN_DIR',cgi_bin_dir,'os2environment')

if addon_dir=0 then addon_dir=servdir||'\ADDON'
if dosisdir(addon_dir)=0 then do
  say "|sreLite ERROR: no such  addon directory: " addon_dir
  "string ERROR: no such addon directory: " addon_dir
   exit
end
addon_dir=strip(strip(addon_dir),'t','\')
foo=value('SRELI_ADDON_DIR',addon_dir,'os2environment')

if tempdata_dir=0 then tempdata_dir=servdir||'\TEMP'
if dosisdir(tempdata_dir)=0 then do
  say "|sreLite ERROR: no such tempdata directory: " tempdata_dir
  "string ERROR: no such tempdata directory: " tempdata_dir
  exit
end
tempdata_dir=strip(strip(tempdata_dir),'t','\')
foo=value('SRELI_TEMPDATA_DIR',TEMPDATA_DIR,'os2environment')

ddir=datadir()
ddir=translate(ddir,'\','/')
if tempfile_dir=0 then tempfile_dir=ddir||'TEMP'
if dosisdir(tempfile_dir)=0 then do
  say "|sreLite ERROR: no such tempfile directory: " tempfile_dir
  "string ERROR: no such tempfile directory: " tempfile_dir
  exit
end
tempfile_dir=strip(strip(tempfile_dir),'t','\')
foo=value('SRELI_tempfile_DIR',tempfile_DIR,'os2environment')

if user_file=0 then user_file=servdir||'\data\users.in'
user_file0=stream(user_file,'c','query exists')
if user_file0='' then do
  say "|sreLite Caution: Missing User file: "user_file
  user_file=0
end
foo=value('SRELI_USERFILE',user_file,'os2environment')

if common_log_file=0 then common_log_file=servdir||'\common.log'

thid=start_thread('USER','SRELITE1.RXX',enmadd,servdir,user_file)
say "|sreLite: User thread on thread# : "thid

if suppress_varstore<>1 then do
   thid2=start_thread('VARSTORE','SRELITE2.RXX',enmadd,servdir)
   say "|sreLite: VARSTORE thread on thread# : "thid2
end


foo=value('SREF_PREFIX',enmadd,'os2environment')

foo=value('SREF_DIGEST_AUTH',digest_auth,'os2environment')

foo=value('sreLite_INIT',1,'os2environment')

foo=value('SRELI_SERVDIR',servdir,'os2environment')

foo=value('SREF_USEHTTP_10',1,'os2environment')


foo=value('SRELI_COMMON_LOG',common_log_file,'os2environment')

foo=value('SRELI_SUPPRESS_VARSTORE',suppress_varstore,'os2environment')

foo=value('SRELI_EMPTY_PATH_OK',empty_path_ok,'os2environment')

foo=value('SRELI_ENABLE_HTTP11',enable_http11,'os2environment')

foo=value('SRELI_PROXY_CACHE',proxy_cache,'os2environment')


foo=value('SRELI_ADD_SLASH',add_slash,'os2environment')


foo=value(enmadd||'INTERPRET_TYPES',interpret_types,'os2environment')


/*  =========== jump here on a reset */
step1: nop

foo=value('SREF_VERBOSE',verbose,'os2environment')

foo=value('SRELI_ACCEPT_RANGE',accept_range,'os2environment')

/* Read SRELITE.INI */
inifile=servdir'\SRELITE.INI'
inf=stream(inifile,'c','query size')
if inf=0 | inf='' then do
  say "|sreLite ERROR: Missing Initialization file: "inifile
  string "Missing Initialization File: " inifile
  exit
end /* do */
foo=value('SRELI_INITFILE',inifile,'os2environment')

inhouseips.0=0 ; sel_requires.0=0 ; aliases.0=0
foo=stream(inifile,'c','open read')
stuff=charin(inifile,1,inf)
foo=stream(inifile,'c','close')
crlf='0d0a'x
NPS=0
do forever
   if stuff='' then leave
   parse var stuff aline (crlf) stuff
   aline=strip(aline)
   if aline='' then iterate
   if abbrev(aline,';')=1 then iterate
   parse var aline  avar '=' aval
   avar=space(translate(avar),0)
   aval=strip(aval)
   if avar='' then iterate      /* bad value */
   if aval='' then aval="' '"
   a1=left(aval,1)
   if a1='"' | a1="'" then aval=strip(aval,,a1)
   foo=value('SRELI_'||avaR,aval,'os2environment')
   NPS=NPS+1
end

if candogzip=0 then foo=value('SRELI_GZIP_THESE','0','os2environment')

aa=" #parameters, #sel_requires, aliases, InHouseIPS= "nps', 'sel_requires.0', 'aliases.0', 'inhouseips.0
call pmprintf('|sreLite: 'aa)
return aa

/***************************/
/* start a thread */
start_thread:procedure
parse arg tname,progfile,enmadd,servdir,extrainfo
tname=strip(translate(tname))

arxx=servdir||'\'||progfile
usequeue=enmadd||tname
usesem='\SEM32\'||enmadd||tname
a=rxqueue('d',usequeue)
a=rxqueue('c',usequeue)
if a<>usequeue then do
  bb=rxqueue('d',a)
  a=rxqueue('s',usequeue)
  do queued()
       pull
  end /* do */
end
a=eventsem_close(usesem)
A=EVENTSEM_CREATE(USESEM)
IF A<>0 then DO
    say '|sreLite ERROR: creating 'tname' semaphore: 'A' 'usesem
    'string  Error creating 'tname' semaphore: 'A' 'usesem
    exit
end
a_tid=rexxthread('f',arxx,usequeue,USESEM,15000,extrainfo)
if a_tid=0 then do
  say "|sreLite ERROR: Could not open 'tname' thread: "arxx
  string "Error opening 'tname' thread "
  exit
end
return a_tid


/*** some basic initializations */
init1:
servdir=directory()
servdir=strip(servdir,'t','\')
enmadd='SREF_'||port||'_'
return 1
